@@ -31,6 +31,7 @@
 
 #include "gsearchtool-support.h"
 #include "gsearchtool-callbacks.h"
+#include "gsearchtool.h"
 
 #include <regex.h>
 #include <gtk/gtkcheckbutton.h>
@@ -44,9 +45,11 @@
 #include <gconf/gconf-client.h>
 #include <gconf/gconf.h>
 
-#define PREF_DESKTOP_ICON_THEME "/desktop/gnome/file_views/icon_theme"
 #define C_STANDARD_STRFTIME_CHARACTERS "aAbBcdHIjmMpSUwWxXyYZ"
 #define C_STANDARD_NUMERIC_STRFTIME_CHARACTERS "dHIjmMSUwWyY"
+#define BINARY_EXEC_MIME_TYPE      "application/x-executable-binary"
+#define ICON_THEME_DIRECTORY_ICON  "gnome-fs-directory"
+#define ICON_THEME_EXECUTABLE_ICON "gnome-fs-executable"
 
 /* START OF THE CDDB FUNCTIONS */
 
@@ -645,127 +648,54 @@ gchar *
 get_file_type_with_mime_type (const gchar *filename, 
 			      const gchar *mimetype)
 {
-	if (mimetype == NULL) {
-		return NULL;
+	if (filename == NULL || mimetype == NULL) {
+		return (gchar *)gnome_vfs_mime_get_description (GNOME_VFS_MIME_TYPE_UNKNOWN);
 	}
 
-	if (g_ascii_strcasecmp (mimetype, GNOME_VFS_MIME_TYPE_UNKNOWN) == 0) {
-		if (g_file_test (filename, G_FILE_TEST_IS_DIR))
-			return _("folder");
-		if (g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE))
-			return _("program");	
-		if (g_file_test (filename, G_FILE_TEST_IS_SYMLINK))
-			return _("symbolic link");
+	if (g_file_test (filename, G_FILE_TEST_IS_SYMLINK) == TRUE) {
+		if (g_ascii_strcasecmp (mimetype, GNOME_VFS_MIME_TYPE_UNKNOWN) == 0) {
+			GnomeVFSFileInfo *file_info;
+		
+			file_info = gnome_vfs_file_info_new ();
+			gnome_vfs_get_file_info (filename, file_info, GNOME_VFS_FILE_INFO_DEFAULT);
+		
+			if (g_file_test (file_info->symlink_name, G_FILE_TEST_EXISTS) != TRUE) {
+				gnome_vfs_file_info_unref (file_info);
+				return _("link (broken)");
+			}
+			
+			gnome_vfs_file_info_unref (file_info);
+		}
+		return g_strdup_printf (_("link to %s"), 
+			(gchar *)gnome_vfs_mime_get_description (mimetype));
 	}
 
 	return (gchar *)gnome_vfs_mime_get_description (mimetype);
 } 
 
-static gchar *
-get_mime_name (const gchar *mime_type, 
-	       gint         size)
-{
-	gchar *mime_type_modified;
-	gchar *first_slash;
-	gchar *mime_name;
-
-	mime_type_modified = g_strdup (mime_type);
-	first_slash = strchr (mime_type_modified, '/');
-	if (first_slash != NULL)
-		*first_slash = '-';
-
-	if (size > 0)
-		mime_name = g_strdup_printf ("gnome-%s-%d", 
-					     mime_type_modified, 
-					     size);
-	else
-		mime_name = g_strconcat ("gnome-",
-					 mime_type_modified,
-					 NULL);
-	g_free (mime_type_modified);
-
-	return mime_name;
-}
-
 gchar *
 get_file_icon_with_mime_type (const gchar *filename, 
 			      const gchar *mimetype) 
 {
-	const gchar *icon_name;
-	gchar *mime_name = NULL;	
+	const GnomeIconData *icon_data;
+	gchar *icon_name = NULL;	
 	gchar *icon_path = NULL;
-	gchar *name_with_ext;
-	gchar *tmp;
-	gchar *icon_theme = gsearchtool_gconf_get_string (PREF_DESKTOP_ICON_THEME);
-	
-	icon_name = gnome_vfs_mime_get_icon (mimetype);
-	
-	if (g_file_test ((filename), G_FILE_TEST_IS_DIR) == TRUE) {
-		icon_name = g_strdup ("i-directory");
-	}
-	
-	if (icon_name == NULL) {
-		mime_name = get_mime_name (mimetype, 0);
-		icon_name = mime_name;
-	}
+	int base_size;
 
-	if (!file_extension_is (icon_name, ".png")) {
-		name_with_ext = g_strconcat (icon_name, ".png", NULL);
+	if (g_file_test (filename, G_FILE_TEST_IS_DIR) == TRUE) {
+		icon_name = g_strdup (ICON_THEME_DIRECTORY_ICON);
+	} 
+	else if ((g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE) == TRUE) &&
+		 (g_ascii_strcasecmp (mimetype, "application/x-executable-binary") == 0)) {
+		icon_name = g_strdup (ICON_THEME_EXECUTABLE_ICON);
 	}
 	else {
-		name_with_ext = g_strdup (icon_name);
-	}
-	
-        if (icon_path == NULL) {
-                tmp = g_strconcat ("nautilus/", 
-				   icon_theme, 
-				   "/", 
-				   name_with_ext, 
-				   NULL);
-		icon_path = gnome_vfs_icon_path_from_filename (tmp);
-		g_free (tmp);
-        }
-
-	if (icon_path == NULL) {
-                tmp = g_strconcat ("document-icons/", 
-				   name_with_ext, 
-				   NULL);
-                icon_path = gnome_vfs_icon_path_from_filename (tmp);
-                g_free (tmp);
+		icon_name = gnome_icon_lookup (interface.icon_theme, NULL, filename, NULL, 
+					       NULL, mimetype, 0, NULL);
 	}
 
-	if (icon_path == NULL) {
-		tmp = g_strconcat (g_get_home_dir (),
-				   "/.nautilus/themes/",
-				   icon_theme, 
-				   "/", 
-				   name_with_ext, 
-				   NULL);
-		icon_path = gnome_vfs_icon_path_from_filename (tmp);
-		g_free (tmp);
-	}
-
-        if (icon_path == NULL) {
-		tmp = g_strconcat ("nautilus/", 
-				   icon_theme, 
-				   "/", 
-				   "i-regular.png", 
-				   NULL);
-		icon_path = gnome_vfs_icon_path_from_filename (tmp);
-		g_free (tmp);
-	}
-	
-	if (icon_path == NULL) {
-		tmp = g_strconcat ("document-icons/",
-				   "i-regular.png", 
-				   NULL);
-		icon_path = gnome_vfs_icon_path_from_filename (tmp);
-		g_free (tmp);
-	}
-	
-	g_free (name_with_ext);
-	g_free (icon_theme);
-
+	icon_path = gnome_icon_theme_lookup_icon (interface.icon_theme, icon_name, ICON_SIZE, 
+						  &icon_data, &base_size);
 	return icon_path;	
 }
 
@@ -788,6 +718,20 @@ is_nautilus_running (void)
 	return ret;
 }
 
+gboolean
+is_component_action_type (const gchar *filename)
+{
+	gchar *mimeType = gnome_vfs_get_mime_type (filename);	
+	GnomeVFSMimeActionType actionType = gnome_vfs_mime_get_default_action_type (mimeType);
+	
+	g_free (mimeType);
+	
+	if (actionType == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
+		return TRUE;
+	}
+	return FALSE;
+}
+
 gboolean
 open_file_with_nautilus (const gchar *filename)
 {
@@ -807,26 +751,47 @@ open_file_with_nautilus (const gchar *filename)
 }
 
 gboolean
-open_file_with_application(const gchar *filename)
+open_file_with_application (const gchar *filename)
 {
-	const char *mimeType = gnome_vfs_mime_type_from_name(filename);	
-	GnomeVFSMimeApplication *mimeApp = gnome_vfs_mime_get_default_application(mimeType);
+	gchar *mimeType = gnome_vfs_get_mime_type (filename);	
+	GnomeVFSMimeApplication *mimeApp = gnome_vfs_mime_get_default_application (mimeType);
+	
+	g_free (mimeType);
 		
 	if (mimeApp) {
-		char **argv;
-		int argc = 2;
-
-		argv = g_new0 (char *, argc);
-		argv[0] = mimeApp->command;
-		argv[1] = (gchar *)filename;	
+		gint  argc;
+		gchar **argv;
+		gchar *command_line;
 		
-		if (mimeApp->requires_terminal) 
+		command_line = g_strdup_printf("%s '%s'", mimeApp->command, 
+			escape_single_quotes (filename));
+		
+		g_shell_parse_argv (command_line, &argc, &argv, NULL); 
+			
+		if (mimeApp->requires_terminal) {
 			gnome_prepend_terminal_to_vector(&argc, &argv);	
+		}
 		
 		gnome_execute_async(NULL, argc, argv);	
-		gnome_vfs_mime_application_free(mimeApp);	
+		gnome_vfs_mime_application_free(mimeApp);
+		g_free (command_line);	
 		g_free(argv);
 		return TRUE;
 	}
 	return FALSE;
 }
+
+gboolean
+launch_file (const gchar *filename)
+{
+	gchar *mime_type = gnome_vfs_get_mime_type (filename);	
+	gboolean result = FALSE;
+	
+	if ((g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE) == TRUE) &&
+	    (g_ascii_strcasecmp (mime_type, BINARY_EXEC_MIME_TYPE) == 0)) { 
+		result = g_spawn_command_line_async (filename, NULL);
+	}
+	
+	g_free (mime_type);
+	return result;
+}
